import os
import sys
import pipes
import logging
import subprocess
from typing import Dict, Any
from pathlib import Path

from .utils import get_env_dir


log: logging.Logger = logging.getLogger(__name__)

ENV_DIR: Path = get_env_dir()
BINARIES_DIR: Path = ENV_DIR / 'bin'


def ensure_available() -> Path:
    """Ensure the node executable is available

    Returns the path of the bin/ directory containing node
    """
    if not ENV_DIR.exists():
        _install_node_env()
    else:
        log.debug('Environment exists at %s', ENV_DIR)

    # Ensure all the binaries we require are installed.
    # This shouldn't really happen but there could
    # be cases where our env dir exists but without the
    # binaries so we might as well just double check.
    for binary in ['node', 'npm', 'npx']:
        if not BINARIES_DIR.joinpath(binary).exists():
            _install_node_env()

    path = BINARIES_DIR.joinpath('node')
    if not path.exists():
        raise RuntimeError(f'Expected node to exist at {path}')
    return BINARIES_DIR


def _install_node_env() -> None:
    args = [sys.executable, '-m', 'nodeenv', str(ENV_DIR)]
    log.debug('Running command with args: %s', args)
    subprocess.run(args, check=True)


def run(*args: str) -> int:
    ensure_available()

    activate = ENV_DIR / 'bin' / 'activate'
    new_args = ('bash', '-c', f'. {pipes.quote(str(activate))} && {" ".join(args)}')
    log.debug('Running node command with args: %s', new_args)

    proc = subprocess.run(new_args, env={**os.environ, **get_env_variables()})
    return proc.returncode


def get_env_variables() -> Dict[str, Any]:
    """Return the environmental variables that should be passed to a binary"""
    # NOTE: I do not actually know if these result in the intended behaviour
    #       I simply copied them from bin/shim in nodeenv
    return {
        'NODE_PATH': str(ENV_DIR / 'lib' / 'node_modules'),
        'NPM_CONFIG_PREFIX': str(ENV_DIR),
        'npm_config_prefix': str(ENV_DIR),
    }



import os
import sys
import logging
from typing import List, NoReturn

from . import node


log: logging.Logger = logging.getLogger(__name__)
PYRIGHT_VERSION: str = os.environ.get('PYRIGHT_PYTHON_VERSION', '1.1.150')


def main(args: List[str]) -> int:
    return node.run('npx', '--yes', f'pyright@{PYRIGHT_VERSION}', *args)


def entrypoint() -> NoReturn:
    sys.exit(main(sys.argv[1:]))



import os
import tempfile
from pathlib import Path

from . import __version__


def get_env_dir() -> Path:
    """Returns the directory that contains the nodeenv"""
    env_dir = os.environ.get('PYRIGHT_PYTHON_ENV_DIR')
    if env_dir is not None:
        return Path(env_dir)

    return Path(tempfile.gettempdir()) / 'pyright-prisma' / __version__ / 'env'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import re
from setuptools import setup


with open('README.md', 'r') as f:
    readme = f.read()

version = ''
with open('pyright/__init__.py') as f:
    match = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE)
    if not match:
        raise RuntimeError('version is not set')

    version = match.group(1)

if not version:
    raise RuntimeError('version is not set')


with open('requirements.txt', 'r') as f:
    requirements = f.readlines()


extras = {
    'dev': [
        'twine>=3.4.1',
    ]
}

setup(
    name='pyright',
    version=version,
    author='Robert Craigie',
    maintainer='Robert Craigie',
    license='MIT',
    url='https://github.com/RobertCraigie/pyright-python',
    description='Command line wrapper for pyright',
    install_requires=requirements,
    long_description=readme,
    long_description_content_type='text/markdown',
    packages=['pyright'],
    python_requires='>=3',
    package_data={'': ['py.typed']},
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'pyright=pyright.cli:entrypoint',
            'pyright-python=pyright.cli:entrypoint',
        ],
    },
    extras_require={
        **extras,
        'all': [req for requirements in extras.values() for req in requirements],
    },
    project_urls={
        'Source': 'https://github.com/RobertCraigie/pyright-python',
        'Tracker': 'https://github.com/RobertCraigie/pyright-python/issues',
    },
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'Typing :: Typed',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
